﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.IO;

namespace SystemLead.Framework
{
    /// <summary>
    /// 提供字串加解密服務
    /// </summary>
    public static class CryptoService
    {
        /// <summary>
        /// 隨機字串產生器
        /// </summary>
        /// <param name="length">密碼長度</param>
        /// <returns>隨機字串</returns>
        public static string Generator(int length)
        {
            StringBuilder sb = new StringBuilder();
            Random rad = new Random();
            for (int i = 0; i < length; i++)
            {
                switch (rad.Next(1, 4))
                {
                    case 1: //數字0~9
                        sb.Append(rad.Next(0, 10));
                        break;
                    case 2: //小寫英文a~z
                        sb.Append((char)rad.Next(97, 123));
                        break;
                    case 3: //大小寫英文A~Z
                        sb.Append((char)rad.Next(65, 91));
                        break;
                }
            }
            return sb.ToString();
        }

        /// <summary>
        ///將字串加密
        /// </summary>
        /// <param name="uncodedString">傳入要加密的字串</param>
        /// <returns>加密過的字串</returns>
        public static string Encrypt(string uncodedString)
        {
            if (uncodedString == String.Empty || uncodedString == null) return String.Empty;

            int ibase = GetBase(uncodedString); //取得編碼的基碼
            string lspasswd = AddZero(ToChar(ibase)); // 設定基碼

            ASCIIEncoding ascii = new ASCIIEncoding();

            int li_char = ascii.GetBytes(uncodedString.Substring(0, 1))[0] + ibase;  //取的密碼的第一個字元的 ascii code 加上基碼

            lspasswd += AddZero(ToChar(li_char));

            for (int i = 2; i <= uncodedString.Length; i++)
            {
                //依序將密碼轉換
                if (i % 2 == 0)
                {
                    li_char = ascii.GetBytes(uncodedString.Substring(i - 1, 1))[0] + (ibase + i);
                }
                else
                {
                    li_char = ascii.GetBytes(uncodedString.Substring(i - 1, 1))[0] - (ibase + i);
                }

                lspasswd += AddZero(ToChar(li_char));
            }

            return lspasswd.Trim().ToString();
        }

        /// <summary>
        /// 將字串解密
        /// </summary>
        /// <param name="encodedString">要解密的字串</param>
        /// <returns>解密過的字串</returns>
        public static string Decrypt(string encodedString)
        {
            if (encodedString == null || encodedString == String.Empty)
                return String.Empty;

            for (int i = 0; i < encodedString.Length; i++)
            {
                if (!Char.IsNumber(encodedString, i))
                {
                    return encodedString;
                }
            }

            int ibase = transbase(encodedString.Substring(0, 3)); //取得編碼的基碼
            int li_char = System.Convert.ToInt32(encodedString.Substring(3, 3)) - ibase; //取的編碼密碼的第4個~第7各字元 ,減掉基碼
            string lspasswd = System.Convert.ToChar(ToChar(li_char)).ToString(); // 記錄密碼


            for (int i = 2; i <= (encodedString.Length / 3) - 1; i++)
            {
                int code = System.Convert.ToInt32(encodedString.Substring(i * 3, 3));
                //依序將密碼轉換
                if (i % 2 == 0)
                {
                    lspasswd += System.Convert.ToChar(ToChar(code - (ibase + i))).ToString();
                }
                else
                {
                    lspasswd += System.Convert.ToChar(ToChar(code + (ibase + i))).ToString();
                }
            }

            return lspasswd.Trim().ToString();


        }

        /// <summary>
        /// 轉基碼字串為數字(000 000 000 000)
        /// </summary>
        /// <param name="baseString"></param>
        /// <returns></returns>
        static string AddZero(int baseString)
        {
            string zeroString = null;

            switch (baseString.ToString().Length)
            {
                case 3:
                    zeroString = baseString.ToString();
                    break;
                case 2:
                    zeroString = "0" + baseString.ToString();
                    break;
                case 1:
                    zeroString = "00" + baseString.ToString();
                    break;

            }

            return zeroString;
        }

        /// <summary>
        /// 轉基碼字串為數字
        /// </summary>
        /// <param name="baseString"></param>
        /// <returns></returns>
        static int transbase(string baseString)
        {
            return System.Convert.ToInt32(baseString);
        }
        /// <summary>
        /// 處理超過 128 char
        /// </summary>
        /// <param name="chr"></param>
        /// <returns></returns>
        static int ToChar(int chr)
        {
            int iChar;
            iChar = chr % 256;

            if (chr > 0)
            {
                return iChar;
            }
            else
            {
                return (256 - System.Convert.ToByte(Math.Abs(iChar)));
            }
        }

        /// <summary>
        /// 取基碼數字
        /// </summary>
        /// <param name="password"></param>
        /// <returns></returns>
        static int GetBase(string password)
        {
            int iNum = 0;

            ASCIIEncoding ascii = new ASCIIEncoding();
            byte[] encodedBytes = ascii.GetBytes(password);

            foreach (byte b in encodedBytes)
            {
                iNum += System.Convert.ToInt32(b);
            }

            return 32 + System.Convert.ToInt32(DateTime.Now.Second % 10) + System.Convert.ToInt32(iNum / password.Length);
        }
        /// <summary>
        /// AES字串加密(非對稱式)
        /// </summary>
        /// <param name="Source">加密前字串</param>
        /// <param name="CryptoKey">加密金鑰</param>
        /// <returns>加密後字串</returns>
        public static string AesEncryptBase64(string SourceStr, string CryptoKey)
        {
            string encrypt = "";
            try
            {
                AesCryptoServiceProvider aes = new AesCryptoServiceProvider();
                MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider();
                SHA256CryptoServiceProvider sha256 = new SHA256CryptoServiceProvider();
                byte[] key = sha256.ComputeHash(Encoding.UTF8.GetBytes(CryptoKey));
                byte[] iv = md5.ComputeHash(Encoding.UTF8.GetBytes(CryptoKey));
                aes.Key = key;
                aes.IV = iv;

                byte[] dataByteArray = Encoding.UTF8.GetBytes(SourceStr);
                using (MemoryStream ms = new MemoryStream())
                using (CryptoStream cs = new CryptoStream(ms, aes.CreateEncryptor(), CryptoStreamMode.Write))
                {
                    cs.Write(dataByteArray, 0, dataByteArray.Length);
                    cs.FlushFinalBlock();
                    encrypt = Convert.ToBase64String(ms.ToArray());
                }
            }
            catch (Exception e)
            {
                throw new ArgumentException(e.Message);
            }
            return encrypt;
        }

        /// <summary>
        /// AES字串解密(非對稱式)
        /// </summary>
        /// <param name="Source">解密前字串</param>
        /// <param name="CryptoKey">解密金鑰</param>
        /// <returns>解密後字串</returns>
        public static string AesDecryptBase64(string SourceStr, string CryptoKey)
        {
            string decrypt = "";
            try
            {
                AesCryptoServiceProvider aes = new AesCryptoServiceProvider();
                MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider();
                SHA256CryptoServiceProvider sha256 = new SHA256CryptoServiceProvider();
                byte[] key = sha256.ComputeHash(Encoding.UTF8.GetBytes(CryptoKey));
                byte[] iv = md5.ComputeHash(Encoding.UTF8.GetBytes(CryptoKey));
                aes.Key = key;
                aes.IV = iv;

                byte[] dataByteArray = Convert.FromBase64String(SourceStr);
                using (MemoryStream ms = new MemoryStream())
                {
                    using (CryptoStream cs = new CryptoStream(ms, aes.CreateDecryptor(), CryptoStreamMode.Write))
                    {
                        cs.Write(dataByteArray, 0, dataByteArray.Length);
                        cs.FlushFinalBlock();
                        decrypt = Encoding.UTF8.GetString(ms.ToArray());
                    }
                }
            }
            catch (Exception e)
            {
                throw new ArgumentException(e.Message);
            }
            return decrypt;
        }

    }
}
