﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations;
using System.Runtime.CompilerServices;

namespace SystemLead.Project.LK.Server.BZSWeb.Models.MIS
{
    #region MisUnit
    public class MisUnit : ICloneable
    {
        [Display(Name = "事業單位代號")]
        [Key]
        [Required(ErrorMessage = "事業單位代號不可為空白"), MaxLength(64)]
        public string FacNo { get; set; }
        [Display(Name = "事業單位名稱")]
        [Required(ErrorMessage = "事業單位名稱不可為空白"), MaxLength(255)]
        public string FacName { get; set; }
        #region Constructor
        public MisUnit() { }
        public MisUnit(string facno,
            string facname)
        {
            FacNo = facno;
            FacName = facname;
        }
        #endregion
        #region 以下為程式建立記憶體資料
        public MisUnit Clone()
        {
            return ((ICloneable)this).Clone() as MisUnit;
        }
        //實作強型別ICloneable介面
        object ICloneable.Clone()
        {
            return this.MemberwiseClone();//使用淺層複製的方法.產出相同屬性值的物件
        }
        #endregion
    }
    #endregion
    #region MisDept
    public class MisDept : ICloneable
    {
        [Display(Name = "事業單位代號")]
        [Key]
        [Required(ErrorMessage = "事業單位代號不可為空白"), MaxLength(64)]
        public string FacNo { get; set; }
        [Display(Name = "部門代號")]
        [Key]
        [Required(ErrorMessage = "部門代號不可為空白"), MaxLength(64)]
        public string DepNo { get; set; }
        [Display(Name = "部門名稱")]
        [Required(ErrorMessage = "部門名稱不可為空白"), MaxLength(255)]
        public string DepName { get; set; }
        public string UpLevel { get; set; }
        #region Constructor
        public MisDept() { }
        public MisDept(string facno,
            string depno,
            string depname,
            string upleave)
        {
            FacNo = facno;
            DepNo = depno;
            DepName = depname;
            UpLevel = upleave;
        }
        #endregion
        #region 以下為程式建立記憶體資料
        public MisDept Clone()
        {
            return ((ICloneable)this).Clone() as MisDept;
        }
        //實作強型別ICloneable介面
        object ICloneable.Clone()
        {
            return this.MemberwiseClone();//使用淺層複製的方法.產出相同屬性值的物件
        }
        #endregion
    }
    #endregion
    #region MisCkind
    public class MisCkind : ICloneable
    {
        [Display(Name = "類別代號")]
        [Key]
        [Required(ErrorMessage = "類別代號不可為空白"), MaxLength(2)]
        public string Ckind { get; set; }
        [Display(Name = "大類名稱")]
        [Required(ErrorMessage = "大類名稱不可為空白"), MaxLength(30)]
        public string Ckdesc { get; set; }
        #region Constructor
        public MisCkind() { }
        public MisCkind(string ckind,
            string ckdesc)
        {
            Ckind = ckind;
            Ckdesc = ckdesc;
        }
        #endregion
        #region 以下為程式建立記憶體資料
        public MisCkind Clone()
        {
            return ((ICloneable)this).Clone() as MisCkind;
        }
        //實作強型別ICloneable介面
        object ICloneable.Clone()
        {
            return this.MemberwiseClone();//使用淺層複製的方法.產出相同屬性值的物件
        }
        #endregion
    }
    #endregion
    #region MisCode
    public class MisCode : ICloneable
    {
        [Display(Name = "類別代號")]
        [Required(ErrorMessage = "類別代號不可為空白"), MaxLength(2)]
        public string Ckind { get; set; }
        [Display(Name = "代碼")]
        [Required(ErrorMessage = "代碼不可為空白"), MaxLength(20)]
        public string Code { get; set; }
        [Display(Name = "代碼名稱")]
        [Required(ErrorMessage = "代碼名稱不可為空白"), MaxLength(60)]
        public string Cdesc { get; set; }
        [Display(Name = "狀態")]
        [Required(ErrorMessage = "狀態不可為空白"), MaxLength(60)]
        public string Status { get; set; }
        public string Cdesc_s { get; set; }
        public string Cdesc_e { get; set; }
        public string Cdesc_j { get; set; }
        public string Cdesc_o { get; set; }
        public string Memo { get; set; }
        public MisCode() { }
        public MisCode(string ckind,
            string code,
            string cdesc,
            string status,
            string cdesc_s,
            string cdesc_e,
            string cdesc_j,
            string cdesc_o,
            string memo
            )
        {
            Ckind = ckind;
            Code = code;
            Cdesc = cdesc;
            Status = status;
            Cdesc_s = cdesc_s;
            Cdesc_e = cdesc_e;
            Cdesc_j = cdesc_j;
            Cdesc_o = cdesc_o;
            Memo = memo;
        }
        #region 以下為程式建立記憶體資料
        public MisCode Clone()
        {
            return ((ICloneable)this).Clone() as MisCode;
        }
        //實作強型別ICloneable介面
        object ICloneable.Clone()
        {
            return this.MemberwiseClone();//使用淺層複製的方法.產出相同屬性值的物件
        }
        #endregion
    }
    #endregion
    #region MisGrp
    public class MisGrp : ICloneable, INotifyCompletion
    {
        [Display(Name = "群組代號")]
        [Key]
        [Required(ErrorMessage = "群組代號不可為空白"), MaxLength(2)]
        public string GrpNo { get; set; }
        [Display(Name = "群組名稱")]
        [Required(ErrorMessage = "群組名稱不可為空白"), MaxLength(30)]
        public string GrpName { get; set; }
        public string c_GrpNoName { get; set; }
        public string ParentId { get; set; }
        public bool Expanded { get; set; }
        public bool HasSubFolders { get; set; }
        public Dictionary<string, Object> htmlAttribute { get; set; }
        public bool IsCompleted { get; }
        public void GetResult() { }
        public void OnCompleted(Action continuation) { }

        #region Constructor
        public MisGrp() { }
        public MisGrp(string grpno, string grpname, string c_grpnoname)
        {
            GrpNo = grpno;
            GrpName = grpname;
            c_GrpNoName = c_grpnoname;
        }
        #endregion
        #region 以下為程式建立記憶體資料
        public MisGrp Clone()
        {
            return ((ICloneable)this).Clone() as MisGrp;
        }
        //實作強型別ICloneable介面
        object ICloneable.Clone()
        {
            return this.MemberwiseClone();//使用淺層複製的方法.產出相同屬性值的物件
        }
        #endregion

        #endregion
        public class CodeName
        {
            public string Code { get; set; }
            public string Name { get; set; }

            public CodeName(string code, string name)
            {
                Code = code;
                Name = name;
            }
        }
    }
}
