using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazored.Modal;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using SystemLead.Project.LK.Server.BZSWeb.Models;
using SystemLead.Project.LK.Server.BZSWeb.Services.MIS;
using SystemLead.Project.LK.Server.BZSWeb.Services;
#region o̱N|Os[JRWŶŧi
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Http;
using System.Net.Http;
using MatBlazor;
using System.Net.Http.Headers;
using System.Text.Encodings.Web;
using System.Text.Unicode;
using Syncfusion.Blazor;
using System.Globalization;
using Microsoft.AspNetCore.Localization;
using SystemLead.Project.LK.Server.BZSWeb.Shared;
using Microsoft.Extensions.Options;
#endregion

namespace SystemLead.Project.LK.Server.BZSWeb
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit https://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {


            services.AddRazorPages();
            services.AddServerSideBlazor();

            #region [Jϥ Cookie {һݭnŧi
            services.Configure<CookiePolicyOptions>(options =>
            {
                options.CheckConsentNeeded = context => true;
                options.MinimumSameSitePolicy = Microsoft.AspNetCore.Http.SameSiteMode.None;
            });
            services.AddAuthentication(
                CookieAuthenticationDefaults.AuthenticationScheme)
                .AddCookie();
            #endregion

            #region [J|Ψ쪺Aȫŧi
            services.AddHttpContextAccessor();
            services.AddScoped<HttpContextAccessor>();
            services.AddHttpClient();
            services.AddScoped<HttpClient>();

            #endregion

            //Inject Menuservice
            services.AddSingleton<MenuService>();
            // 1.EF to DB
            //services.AddScoped<IMisService, MisDbEFService>();
            // 2.Dapper to DB
            services.AddScoped<IMisService, MisDbDapperService>();

            var sqlConnectionConfig = new SqlConnectionConfiguration(Configuration.GetConnectionString("SqlDbContext"));
            services.AddSingleton(sqlConnectionConfig);
            services.AddDbContext<MyDbContext>(options =>
            {
                options.UseSqlServer(sqlConnectionConfig.Value);
            });
            //services.AddDbContext<SchoolContext>(options =>
            //options.UseSqlServer(sqlConnectionConfig.Value), ServiceLifetime.Transient);

            // 3.WebAPI to DB : 
            //var appconfig = new AppConfiguration();
            //appconfig.Uri = Configuration.GetValue<string>("AppStrings:Uri");
            //services.AddHttpClient<IMisService, MisWebAPIService>(client =>
            //{
            //    client.BaseAddress = new Uri(appconfig.Uri);
            //    client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
            //});

            #region Enable Localization 
            services.AddLocalization(options => options.ResourcesPath = "Resources");
            //Syncfusion
            services.AddSyncfusionBlazor();
            // Register the Syncfusion locale service to customize the  SyncfusionBlazor component locale culture
            services.AddSingleton(typeof(ISyncfusionStringLocalizer), typeof(SyncfusionLocalizer));
            services.Configure<RequestLocalizationOptions>(options =>
            {
                // define the list of cultures your app will support
                var supportedCultures = new List<CultureInfo>()
                {
                    new CultureInfo("en-US"),
                    new CultureInfo("zh-TW")
                };
                // set the default culture
                options.DefaultRequestCulture = new RequestCulture("zh");
                options.SupportedCultures = supportedCultures;
                options.SupportedUICultures = supportedCultures;
            });
            #endregion
            services.AddBlazoredModal();

            //MatBlazor CRUD
            services.AddScoped<HttpClient>();

            //MatBlazor Toaster
            services.AddMatToaster(config =>
            {
                config.Position = MatToastPosition.BottomRight;
                config.PreventDuplicates = true;
                config.NewestOnTop = true;
                config.ShowCloseButton = true;
                config.MaximumOpacity = 95;
                config.VisibleStateDuration = 3000;
            });



            services.AddTransient<IHoluserService, HoluserService>();
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                app.UseExceptionHandler("/Error");
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }
            //Mark for Deploy IIS && ngrok Redirect 307 error
            //app.UseHttpsRedirection();
            app.UseStaticFiles();

            #region wnϥ Cookie & ϥΪ̻{Ҫn
            app.UseCookiePolicy();
            app.UseAuthentication();
            #endregion

            app.UseRouting();

            app.UseEndpoints(endpoints =>
            {
                // Adds endpoints for controller actions to the     IEndpointRouteBuilder without specifying any routes.
                endpoints.MapControllers();
                endpoints.MapBlazorHub();
                endpoints.MapFallbackToPage("/_Host");
                #region o̭n[J Web API |Ψ쪺
                endpoints.MapControllerRoute("default", "{controller=Home}/action=Index/{id}");
                #endregion
            });
            #region ŧiҨϥ Syncfusion for Blazor 󪺨ϥαvX
            Syncfusion.Licensing.SyncfusionLicenseProvider.RegisterLicense("MzUwMzUxQDMxMzgyZTMzMmUzMEVweWE5UUlyR3h2SkNyb2pPTitTd0xYci9GaldWQVE3NlllUTQ1STNTMzg9");
            #endregion
            #region Localization
            app.UseRequestLocalization(app.ApplicationServices.GetService<IOptions<RequestLocalizationOptions>>().Value);
            #endregion
        }
    }
}
